/*
    XAMIXER - An X11 Audio Mixer
    Copyright (C) 1995 by Jim MacKinnon
    jmack@IN.Edmonton.AB.CA
    jmack@Phys.UAlberta.CA

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* defs.h - general defs */

#include <linux/soundcard.h>

#define SOUND_FULL_SCALE 100.0

/**** main mixer channel device types - per <linux/soundcard.h>
***** for reference only:

***** 	#define SOUND_MIXER_NRDEVICES   12

***** 	#define SOUND_MIXER_VOLUME      0
***** 	#define SOUND_MIXER_BASS        1
***** 	#define SOUND_MIXER_TREBLE      2

***** 	#define SOUND_MIXER_SYNTH       3
***** 	#define SOUND_MIXER_PCM         4
***** 	#define SOUND_MIXER_SPEAKER     5

***** 	#define SOUND_MIXER_LINE        6
***** 	#define SOUND_MIXER_MIC         7
***** 	#define SOUND_MIXER_CD          8

***** 	#define SOUND_MIXER_IMIX        9 
***** 	#define SOUND_MIXER_ALTPCM      10
***** 	#define SOUND_MIXER_RECLEV      11 

*****/

/* #define DEBUG */
#ifdef DEBUG
#define DPRINTF(x)      printf x
#else
#define DPRINTF(x)
#endif


/* boolean flip-flop macro  and min/max macro */

#define FLIP(x) ((x) = 0 ? (1) : (0) )
#define MIN(a,b) ((a) <= (b) ? (a) : (b))

typedef struct formsobj
{
	u_char	value;		/* value of slider,button, etc */
	FL_OBJECT *obj;		/* pointer to the object itself */
} MyObj;
	
/* this must be left unstructured, since it is passed in ioctl() */
typedef struct stereovolume
{
  u_char left;
  u_char right;
  u_char pad1;
  u_char pad2;
} StereoVolume;

/* to hold floats */
typedef struct stereobal
{
  double left;
  double right;
} StereoBalance;

typedef struct voltoggle
{
        MyObj   lock;		/* whether both L R move together */
        MyObj   solo;		/* mask out all other channels */
        MyObj   Norm;		/* middle position */
        MyObj   max;		/* max value will be set on slider */
        MyObj   mute;           /* doubles as 'min' for some sliders */
        MyObj   rec;		/* this channel is record source */
} VolumeSwitch;

typedef struct volctrl
{
  u_char mixer_id;	/* which sub-device? */
  u_char supported;	/* does hardware work for this type? */
  StereoVolume volume;	/* volume of L R */
  struct {
  FL_OBJECT	*left;
  FL_OBJECT	*right;
  } obj;
  VolumeSwitch toggle;	/* toggle states */
} VolumeControl;

VolumeControl	master, line, dsp, fm, cd, mic, bass, treble, record;
StereoVolume	balance;	/* for master out scaling */
StereoBalance	Sbalance;	/* for master out scaling */

FL_OBJECT 	*center;	/* centering button FM object */
FL_OBJECT 	*bal;		/* balance slider FM object */

FL_OBJECT 	*L_leds[10],*R_leds[10];	/* LED arrays - not functional yet */

StereoVolume	sav_master, sav_balance, \
		sav_line, sav_dsp, sav_fm, sav_cd, sav_mic, \
		sav_bass, sav_treble, sav_record;

/* FILE descriptor types we need */
int mixer_fd;		/* for /dev/mixer */

/*---------------------------------------*/
