 /*
    XAMIXER - An X11 Audio Mixer
    Copyright (C) 1995 by Jim MacKinnon
    jmack@IN.Edmonton.AB.CA
    jmack@Phys.UAlberta.CA

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <sys/errno.h>
#include <fcntl.h>

#include "forms.h"
#include "xamixer.h"

#include "defs.h"

char*           progname;
   const char * dname[] = SOUND_DEVICE_NAMES;
   int devmask, stereod, i, d;
   int lcval[SOUND_MIXER_NRDEVICES];

int main(int argc, char *argv[])
{
   FD_xamixer *fd_xamixer;
   extern void do_setup();

   progname = argv[0];

   fl_initialize(argv[0], "Audio Mixer", 0, 0, &argc, argv);
   fd_xamixer = create_form_xamixer();

   /* Open the mixer device */
   mixer_fd = open ("/dev/mixer", O_RDWR, 0);
   if (mixer_fd < 0)
       perror ("Error opening mixer device"), exit (1);

    if (ioctl(mixer_fd, SOUND_MIXER_READ_DEVMASK, &devmask) == -1) {
	perror("SOUND_MIXER_READ_DEVMASK");
	exit(-1);
    }
    if (ioctl(mixer_fd, SOUND_MIXER_READ_STEREODEVS, &stereod) == -1) {
	perror("SOUND_MIXER_READ_STEREODEVS");
	exit(-1);
    }

    if (!devmask) {
	fprintf(stderr, "No device found.");
	exit(-1);
    }

   do_setup();

   /* fill-in form initialization code */

   /* show the first form */
   fl_show_form(fd_xamixer->xamixer,FL_PLACE_CENTER,FL_FULLBORDER,"Audio Mixer");
   fl_do_forms();

   /* program should never get here */
   return 0;
}


/* initialization */

void do_setup() {

/* default lock settings */
	cd.toggle.lock.value = 1;
	dsp.toggle.lock.value = 1;
	fm.toggle.lock.value = 1;
	line.toggle.lock.value = 1;
	mic.toggle.lock.value = 1;
	record.toggle.lock.value = 1;
	bass.toggle.lock.value = 1;
	treble.toggle.lock.value = 1;
	master.toggle.lock.value = 1;

/* default flag settings */
	cd.toggle.Norm.value = 1;
	dsp.toggle.Norm.value = 1;
	fm.toggle.Norm.value = 1;
	line.toggle.Norm.value = 1;
	mic.toggle.Norm.value = 1;
	record.toggle.Norm.value = 1;
	bass.toggle.Norm.value = 1;
	treble.toggle.Norm.value = 1;
	master.toggle.Norm.value = 1;

	record.toggle.rec.value = 1;

/* default button startup settings */
	fl_set_button(cd.toggle.Norm.obj, 1);
	fl_set_button(dsp.toggle.Norm.obj, 1);
	fl_set_button(fm.toggle.Norm.obj, 1);
	fl_set_button(line.toggle.Norm.obj, 1);

	fl_set_button(bass.toggle.Norm.obj, 1);
	fl_set_button(treble.toggle.Norm.obj, 1);
	fl_set_button(master.toggle.Norm.obj, 1);

	fl_set_button(mic.toggle.mute.obj, 1);
	fl_set_button(record.toggle.mute.obj, 1);

	fl_set_button(record.toggle.rec.obj, 1);

/* default volume levels - loaded in from the device */
/* thanks to Michal Jaegermann in setmixer.c */
	for (i = 0; i < SOUND_MIXER_NRDEVICES; i++) {
	    d = (1 << i);
	    if (0 != (devmask & d)) {
		if (-1 == ioctl(mixer_fd, MIXER_READ(i), &lcval[i])) {
		    perror("MIXER_READ");
		}
	    }
	}

	master.volume.left	= lcval[SOUND_MIXER_VOLUME] & 0x7f ;
	master.volume.right	= (lcval[SOUND_MIXER_VOLUME] >>8) & 0x7f ;
	cd.volume.left		= lcval[SOUND_MIXER_CD] & 0x7f ;
	cd.volume.right		= (lcval[SOUND_MIXER_CD] >>8) & 0x7f ;
	dsp.volume.left		= lcval[SOUND_MIXER_SYNTH] & 0x7f ;
	dsp.volume.right	= (lcval[SOUND_MIXER_SYNTH] >>8) & 0x7f ;
	fm.volume.left		= lcval[SOUND_MIXER_PCM] & 0x7f ;
	fm.volume.right		= (lcval[SOUND_MIXER_PCM] >>8) & 0x7f ;
	line.volume.left	= lcval[SOUND_MIXER_LINE] & 0x7f ;
	line.volume.right	= (lcval[SOUND_MIXER_LINE] >>8) & 0x7f ;

	sav_master.left	 =  master.volume.left;
	sav_master.right =  master.volume.right;
	sav_cd.left	=  cd.volume.left;
	sav_cd.right	=  cd.volume.right;
	sav_dsp.left	=  dsp.volume.left;
	sav_dsp.right	=  dsp.volume.right;
	sav_fm.left	=  fm.volume.left;
	sav_fm.right	=  fm.volume.right;
	sav_line.left	=  line.volume.left;
	sav_line.right  =  line.volume.right;


/* NOT read in from the card (to prevent feeedback/squelch) */
	sav_record.left = sav_record.right = record.volume.left = record.volume.right = 1.0;
	sav_mic.left = sav_mic.right = mic.volume.left = mic.volume.right = 1.0;

/* logical controls */
	sav_balance.left = sav_balance.right = balance.left = balance.right = 0.50;
	Sbalance.left = Sbalance.right = 0.50;

/* set up the sliders in sync with the preset read-in levels */
	fl_set_slider_value(master.obj.left, 1.0 - ( sav_master.left / SOUND_FULL_SCALE) );
	fl_set_slider_value(master.obj.right, 1.0 - (sav_master.right / SOUND_FULL_SCALE) );

	fl_set_slider_value(cd.obj.left, 1.0 - ( sav_cd.left / SOUND_FULL_SCALE) );
	fl_set_slider_value(cd.obj.right, 1.0 - ( sav_cd.right / SOUND_FULL_SCALE) );

	fl_set_slider_value(dsp.obj.left, 1.0 - ( sav_dsp.left / SOUND_FULL_SCALE) );
	fl_set_slider_value(dsp.obj.right, 1.0 - ( sav_dsp.right / SOUND_FULL_SCALE) );

	fl_set_slider_value(fm.obj.left, 1.0 -( sav_fm.left / SOUND_FULL_SCALE) );
	fl_set_slider_value(fm.obj.right,1.0 - ( sav_fm.right / SOUND_FULL_SCALE) );

	fl_set_slider_value(line.obj.left,1.0 -( sav_line.left / SOUND_FULL_SCALE) );
	fl_set_slider_value(line.obj.right, 1.0 - (sav_line.right / SOUND_FULL_SCALE) );

/* make sure record and mic are low */
	fl_set_slider_value(record.obj.left,1.0);
	fl_set_slider_value(record.obj.right,1.0);

	fl_set_slider_value(mic.obj.left,1.0);
	fl_set_slider_value(mic.obj.right,1.0);

/* logical for balance slider */
	fl_set_slider_value(bal,0.50);

}

/* end of do_setup */

/*---------------------------------------*/
